@@ -13,14 +13,19 @@
    }
 
    resid <- unlist( residuals( object ) )
+   resid <- resid[ !is.na( resid ) ]
    nEq   <- length( object$eq )
-   nObsPerEq <- nrow( residuals( object ) )
    if( is.null( object$residCovEst ) ) {
       rcov <- diag( nEq )
    } else {
       rcov <- object$residCovEst
    }
 
+   validObsEq <- matrix( NA, nrow = nrow( residuals( object ) ), ncol = nEq )
+   for( i in 1:nEq ) {
+      validObsEq[ , i ] <- !is.na( residuals( object$eq[[ i ]] ) )
+   }
+
    result <- list()
 
    result$nRestr <- nrow( restrict.matrix )
@@ -30,18 +35,15 @@
       solve( restrict.matrix %*% vcov %*% t( restrict.matrix ) ) %*%
       ( restrict.matrix %*% coef - restrict.rhs )
 
+   resid <- matrix( resid, ncol = 1 )
    if( object$control$useMatrix ) {
-      resid <- matrix( resid, ncol = 1 )
       resid <- as( resid, "dgCMatrix" )
       rcov <- as( rcov, "dspMatrix" )
-
-      denominator <- as.numeric( .calcXtOmegaInv( xMat = resid, sigma = rcov,
-         nObsEq = nObsPerEq, useMatrix = TRUE ) %*% resid )
-   } else {
-      denominator <- crossprod( resid, solve( rcov ) %x% diag( nObsPerEq ) ) %*%
-         resid
    }
 
+   denominator <- as.numeric( .calcXtOmegaInv( xMat = resid, sigma = rcov,
+      validObsEq = validObsEq, useMatrix = object$control$useMatrix ) %*% resid )
+
    result$statistic <- ( numerator / result$nRestr ) /
       ( denominator / result$df.residual.sys )
 
